@@ -27,11 +27,8 @@ struct aes_regwrite {
 	unsigned char value;
 };
 
-struct aes_stripe {
-	int delta_x;
-	int delta_y;
-	unsigned char data[0];
-};
+struct fpi_frame;
+struct fpi_frame_asmbl_ctx;
 
 typedef void (*aes_write_regv_cb)(struct fp_img_dev *dev, int result,
 	void *user_data);
@@ -39,12 +36,10 @@ typedef void (*aes_write_regv_cb)(struct fp_img_dev *dev, int result,
 void aes_write_regv(struct fp_img_dev *dev, const struct aes_regwrite *regs,
 	unsigned int num_regs, aes_write_regv_cb callback, void *user_data);
 
-unsigned int aes_calc_delta(GSList *stripes, size_t stripes_len,
-	unsigned int frame_width, unsigned int frame_height,
-	gboolean reverse);
-
-struct fp_img *aes_assemble(GSList *stripes, size_t stripes_len,
-	unsigned int frame_width, unsigned int frame_height, unsigned int img_width);
+unsigned char aes_get_pixel(struct fpi_frame_asmbl_ctx *ctx,
+			    struct fpi_frame *frame,
+			    unsigned int x,
+			    unsigned int y);
 
 #endif
 
